
from logging import Logger
from logging import getLogger

from xml.etree.ElementTree import Element
from xml.etree.ElementTree import SubElement

from ogl.OglClass import OglClass

from oglio.Types import OglClasses
from oglio.toXmlV11.BaseOglToXml import BaseOglToXml
from oglio.toXmlV11.PyutToXml import PyutToXml

from oglio.toXmlV11.XmlConstants import XmlConstants


class OglClassToXml(BaseOglToXml):

    def __init__(self):
        super().__init__()
        self.logger: Logger = getLogger(__name__)
        self._pyutToXml: PyutToXml = PyutToXml()

    def serialize(self, documentTop: Element, oglClasses: OglClasses) -> Element:

        for oglClass in oglClasses:
            self._oglClassToXml(documentTop=documentTop, oglClass=oglClass)

        return documentTop

    def _oglClassToXml(self, documentTop: Element, oglClass: OglClass) -> Element:
        """
        Exports an OglClass to a minidom Element.

        Args:
            documentTop:     The document to append to
            oglClass:   Ogl Class to serialize

        Returns:
            The newly created `OglClass` Element
        """
        attributes = self._oglBaseAttributes(oglObject=oglClass)
        oglClassSubElement: Element = SubElement(documentTop, XmlConstants.ELEMENT_OGL_CLASS, attrib=attributes)

        self._pyutToXml.pyutClassToXml(graphicElement=oglClassSubElement, pyutClass=oglClass.pyutObject)

        return oglClassSubElement






from typing import Tuple

from logging import Logger
from logging import getLogger
from xml.dom.minidom import Document
from xml.dom.minidom import Element

from ogl.OglObject import OglObject

from oglio.toXmlV10.BaseToDom import BaseToDom
from oglio.toXmlV10.XmlConstants import XmlConstants


class BaseOglToDom(BaseToDom):

    def __init__(self, xmlDocument: Document):

        super().__init__()
        self.baseLogger:   Logger   = getLogger(__name__)
        self._xmlDocument: Document = xmlDocument

    def _appendOglBase(self, oglObject: OglObject, root: Element) -> Element:
        """
        Saves the position and size of the OGL object in an ML node.

        Args:
            oglObject:  OGL Object
            root:      XML node to update

        Returns:
            The updated element
        """
        # Saving size
        w, h = oglObject.GetModel().GetSize()
        simpleW, simpleH = self._getSimpleDimensions(w, h)
        root.setAttribute(XmlConstants.ATTR_WIDTH, simpleW)
        root.setAttribute(XmlConstants.ATTR_HEIGHT, simpleH)

        # Saving position
        x, y = oglObject.GetModel().GetPosition()
        simpleX, simpleY = self._getSimpleCoordinates(x, y)
        root.setAttribute(XmlConstants.ATTR_X, simpleX)
        root.setAttribute(XmlConstants.ATTR_Y, simpleY)

        return root

    def _getSimpleDimensions(self, w: int, h: int) -> Tuple[str, str]:
        # reuse code but not name
        return self._getSimpleCoordinates(w, h)

    def _getSimpleCoordinates(self, x: int, y: int) -> Tuple[str, str]:
        """

        Args:
            x: coordinate
            y: coordinate

        Returns:
            Simple formatted string versions of the above

        """
        simpleX: str = str(int(x))      # some older files used float
        simpleY: str = str(int(y))      # some older files used float

        return simpleX, simpleY




from logging import Logger
from logging import getLogger
from typing import Tuple

from xml.etree.ElementTree import Element
from xml.etree.ElementTree import SubElement

from miniogl.AttachmentSide import AttachmentSide
from miniogl.SelectAnchorPoint import SelectAnchorPoint
from ogl.OglAssociation import OglAssociation
from ogl.OglAssociationLabel import OglAssociationLabel
from ogl.OglInterface2 import OglInterface2
from ogl.OglLink import OglLink

from oglio.Types import OglLinks

from oglio.toXmlV11.PyutToXml import PyutToXml
from oglio.toXmlV11.XmlConstants import XmlConstants
from oglio.toXmlV11.BaseOglToXml import BaseOglToXml
from oglio.toXmlV11.InternalTypes import ElementAttributes


class OglLinksToXml(BaseOglToXml):
    def __init__(self):
        super().__init__()
        self.logger: Logger = getLogger(__name__)

        self._pyutToXml: PyutToXml = PyutToXml()

    def serialize(self, documentTop: Element, oglLinks: OglLinks) -> Element:

        for oglLink in oglLinks:
            if isinstance(oglLink, OglInterface2):
                self._oglInterface2ToXml(documentElement=documentTop, oglInterface=oglLink)
            else:
                self._oglLinkToXml(documentElement=documentTop, oglLink=oglLink)

        return documentTop

    def _oglLinkToXml(self, documentElement: Element, oglLink: OglLink) -> Element:

        attributes:        ElementAttributes = self._oglLinkAttributes(oglLink=oglLink)
        oglLinkSubElement: Element           = SubElement(documentElement, XmlConstants.ELEMENT_OGL_LINK, attrib=attributes)

        if isinstance(oglLink, OglAssociation):

            center: OglAssociationLabel = oglLink.centerLabel
            src:    OglAssociationLabel = oglLink.sourceCardinality
            dst:    OglAssociationLabel = oglLink.destinationCardinality
            associationLabels = {
                XmlConstants.ELEMENT_ASSOCIATION_CENTER_LABEL:      center,
                XmlConstants.ELEMENT_ASSOCIATION_SOURCE_LABEL:      src,
                XmlConstants.ELEMENT_ASSOCIATION_DESTINATION_LABEL: dst
            }
            for eltName in associationLabels:
                oglAssociationLabel: OglAssociationLabel = associationLabels[eltName]

                relativePosition: Tuple[int, int] = oglAssociationLabel.GetRelativePosition()
                x: int = relativePosition[0]
                y: int = relativePosition[1]

                labelAttributes: ElementAttributes = ElementAttributes({
                    XmlConstants.ATTR_X: str(x),
                    XmlConstants.ATTR_Y: str(y),
                })
                # noinspection PyUnusedLocal
                labelElement: Element = SubElement(oglLinkSubElement, eltName, attrib=labelAttributes)

        # save control points (not anchors!)
        for x, y in oglLink.segments[1:-1]:
            controlPointAttributes: ElementAttributes = ElementAttributes({
                XmlConstants.ATTR_X: str(x),
                XmlConstants.ATTR_Y: str(y),
            })
            SubElement(oglLinkSubElement, XmlConstants.ELEMENT_MODEL_CONTROL_POINT, attrib=controlPointAttributes)

        self._pyutToXml.pyutLinkToXml(pyutLink=oglLink.pyutObject, oglLinkElement=oglLinkSubElement)

        return oglLinkSubElement

    def _oglInterface2ToXml(self, documentElement: Element, oglInterface: OglInterface2) -> Element:
        """

        Args:
            documentElement:  untangler element for document
            oglInterface:     Lollipop to serialize
        Returns:
            New untangle element
        """
        destAnchor:      SelectAnchorPoint = oglInterface.destinationAnchor
        attachmentPoint: AttachmentSide    = destAnchor.attachmentPoint
        x, y = destAnchor.GetPosition()

        attributes: ElementAttributes = ElementAttributes({
            XmlConstants.ATTR_LOLLIPOP_ATTACHMENT_POINT: attachmentPoint.__str__(),
            XmlConstants.ATTR_X:                         str(x),
            XmlConstants.ATTR_Y:                         str(y),
        })
        oglInterface2: Element = SubElement(documentElement, XmlConstants.ELEMENT_OGL_INTERFACE2, attrib=attributes)

        self._pyutToXml.pyutInterfaceToXml(oglInterface.pyutInterface, oglInterface2)
        return oglInterface2

    def _oglLinkAttributes(self, oglLink: OglLink) -> ElementAttributes:

        srcX, srcY   = oglLink.sourceAnchor.GetModel().GetPosition()
        destX, destY = oglLink.destinationAnchor.GetModel().GetPosition()

        attributes: ElementAttributes = ElementAttributes({
            XmlConstants.ATTR_LINK_SOURCE_ANCHOR_X:      str(srcX),
            XmlConstants.ATTR_LINK_SOURCE_ANCHOR_Y:      str(srcY),
            XmlConstants.ATTR_LINK_DESTINATION_ANCHOR_X: str(destX),
            XmlConstants.ATTR_LINK_DESTINATION_ANCHOR_Y: str(destY),
            XmlConstants.ATTR_SPLINE:                    str(oglLink.GetSpline())   # piecewise polynomial function
        })

        return attributes
    



from logging import Logger
from logging import getLogger

from ogl.OglObject import OglObject

from oglio.toXmlV10.XmlConstants import XmlConstants


class BaseOglToXml:
    def __init__(self):
        self.logger: Logger = getLogger(__name__)

    def _oglBaseAttributes(self, oglObject: OglObject):
        """
        Create the common OglObject attributes

        Args:
            oglObject:  OGL Object

        Returns:
            The updated originalElement
        """
        w, h = oglObject.GetModel().GetSize()
        x, y = oglObject.GetModel().GetPosition()

        attributes = {
            XmlConstants.ATTR_WIDTH:  str(w),
            XmlConstants.ATTR_HEIGHT: str(h),
            XmlConstants.ATTR_X:      str(x),
            XmlConstants.ATTR_Y:      str(y),
        }

        return attributes



import pathlib
from setuptools import setup
from oglio import __version__ as oglioVersion

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()
LICENSE = (HERE / 'LICENSE').read_text()

setup(
    name="oglio",
    version=oglioVersion,
    author_email='Humberto.A.Sanchez.II@gmail.com',
    maintainer='Humberto A. Sanchez II',
    maintainer_email='humberto.a.sanchez.ii@gmail.com',
    description='External Pyut Persistence',
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/hasii2011/oglio",
    packages=[
        'oglio', 'oglio.toXmlV10', 'oglio.toXmlV11'
    ],
    package_data={
        'oglio':          ['py.typed'],
        'oglio.toXmlV10': ['py.typed'],
        'oglio.toXmlV11': ['py.typed'],
    },
    install_requires=['wxPython==4.2.1', 'codeallybasic==0.5.2', 'codeallyadvanced==0.5.2', 'pyutmodel==1.5.0', 'ogl==0.90.3', 'untanglepyut==1.2.11',],
)




from typing import cast

from logging import Logger
from logging import getLogger

from xml.etree.ElementTree import Element
from xml.etree.ElementTree import SubElement

from ogl.OglNote import OglNote

from oglio.Types import OglNotes

from oglio.toXmlV11.PyutToXml import PyutToXml
from oglio.toXmlV11.XmlConstants import XmlConstants
from oglio.toXmlV11.BaseOglToXml import BaseOglToXml


class OglNotesToXml(BaseOglToXml):
    def __init__(self):
        super().__init__()
        self.logger: Logger = getLogger(__name__)

        self._pyutToXml: PyutToXml = PyutToXml()


    def serialize(self, documentTop: Element, oglNotes: OglNotes) -> Element:

        for note in oglNotes:
            oglNote: OglNote = cast (OglNote, note)
            oglNoteElement: Element = self._oglNoteToXml(documentTop=documentTop, oglNote=oglNote)
            self._pyutToXml.pyutNoteToXml(pyutNote=oglNote.pyutObject, oglNoteElement=oglNoteElement)

        return documentTop

    def _oglNoteToXml(self, documentTop: Element, oglNote: OglNote) -> Element:

        attributes = self._oglBaseAttributes(oglObject=oglNote)
        oglNoteSubElement: Element = SubElement(documentTop, XmlConstants.ELEMENT_OGL_NOTE, attrib=attributes)

        return oglNoteSubElement




from typing import cast

from logging import Logger
from logging import getLogger

from xml.dom.minidom import Document
from xml.dom.minidom import Element

from ogl.OglClass import OglClass

from pyutmodel.PyutClass import PyutClass

from oglio.Types import OglClasses

from oglio.toXmlV10.BaseOglToDom import BaseOglToDom
from oglio.toXmlV10.PyutToDom import PyutToDom
from oglio.toXmlV10.XmlConstants import XmlConstants


class OglClassesToDom(BaseOglToDom):

    def __init__(self, xmlDocument: Document):

        super().__init__(xmlDocument=xmlDocument)
        self.logger: Logger = getLogger(__name__)

        self._pyutToMiniDom: PyutToDom = PyutToDom()

    def serialize(self, documentNode: Element, oglClasses: OglClasses) -> Element:

        for oglClass in oglClasses:
            graphicClass: Element = self._oglClassToDom(oglClass=oglClass, xmlDoc=self._xmlDocument)
            documentNode.appendChild(graphicClass)

        return documentNode

    def _oglClassToDom(self, oglClass: OglClass, xmlDoc: Document) -> Element:
        """
        Exports an OglClass to a minidom Element.

        Args:
            oglClass:   Graphic Class to save
            xmlDoc:     The document to append to

        Returns:
            The newly created `GraphicClass` element
        """
        graphicClass: Element = xmlDoc.createElement(XmlConstants.ELEMENT_GRAPHIC_CLASS)

        graphicClass = self._appendOglBase(oglClass, graphicClass)

        # adding the data layer object
        graphicClass.appendChild(self._pyutToMiniDom.pyutClassToDom(cast(PyutClass, oglClass.pyutObject), xmlDoc))

        return graphicClass




from logging import Logger
from logging import getLogger

from os import linesep as osLineSep

from xml.etree.ElementTree import Element
from xml.etree.ElementTree import SubElement

from pyutmodel.ModelTypes import ClassName
from pyutmodel.PyutActor import PyutActor
from pyutmodel.PyutClass import PyutClass
from pyutmodel.PyutClassCommon import PyutClassCommon
from pyutmodel.PyutField import PyutField
from pyutmodel.PyutInterface import PyutInterface
from pyutmodel.PyutLink import PyutLink
from pyutmodel.PyutMethod import PyutMethod
from pyutmodel.PyutMethod import SourceCode
from pyutmodel.PyutNote import PyutNote
from pyutmodel.PyutParameter import PyutParameter
from pyutmodel.PyutSDInstance import PyutSDInstance
from pyutmodel.PyutSDMessage import PyutSDMessage
from pyutmodel.PyutText import PyutText
from pyutmodel.PyutUseCase import PyutUseCase

from oglio.toXmlV11.BaseXml import BaseXml
from oglio.toXmlV11.InternalTypes import ElementAttributes
from oglio.toXmlV11.XmlConstants import XmlConstants


class PyutToXml(BaseXml):
    """
    Serializes Pyut Models classes to DOM
    """
    # https://www.codetable.net/hex/a
    END_OF_LINE_MARKER: str = '&#xA;'

    def __init__(self):
        super().__init__()
        self.logger: Logger = getLogger(__name__)

    def pyutClassToXml(self, pyutClass: PyutClass, graphicElement: Element) -> Element:
        """
        Exporting a PyutClass to a miniDom Element.

        Args:
            pyutClass:       The pyut class to save
            graphicElement:  The xml element to update

        Returns:
            A new updated element
        """

        commonAttributes = self._pyutClassCommonAttributes(pyutClass)
        attributes = {
            XmlConstants.ATTR_ID:                 str(pyutClass.id),
            XmlConstants.ATTR_NAME:               pyutClass.name,
            XmlConstants.ATTR_STEREOTYPE:         pyutClass.stereotype.value,
            XmlConstants.ATTR_DISPLAY_METHODS:    str(pyutClass.showMethods),
            XmlConstants.ATTR_DISPLAY_PARAMETERS: str(pyutClass.displayParameters),
            XmlConstants.ATTR_DISPLAY_FIELDS:     str(pyutClass.showFields),
            XmlConstants.ATTR_DISPLAY_STEREOTYPE: str(pyutClass.displayStereoType),
        }

        attributes = attributes | commonAttributes

        pyutClassElement: Element = SubElement(graphicElement, XmlConstants.ELEMENT_PYUT_CLASS, attrib=attributes)

        for method in pyutClass.methods:
            self._pyutMethodToXml(pyutMethod=method, pyutClassElement=pyutClassElement)

        for pyutField in pyutClass.fields:
            self._pyutFieldToXml(pyutField=pyutField, pyutClassElement=pyutClassElement)
        return pyutClassElement

    def pyutLinkToXml(self, pyutLink: PyutLink, oglLinkElement: Element) -> Element:
        """
        Exporting a PyutLink to an Element.

        Args:
            pyutLink:   Link to save
            oglLinkElement:     xml document

        Returns:
            A new minidom element
        """
        srcLinkId:  int = self._idFactory.getID(pyutLink.getSource())
        destLinkId: int = self._idFactory.getID(pyutLink.getDestination())

        attributes: ElementAttributes = ElementAttributes({
            XmlConstants.ATTR_NAME:                    pyutLink.name,
            XmlConstants.ATTR_TYPE:                    pyutLink.linkType.name,
            XmlConstants.ATTR_CARDINALITY_SOURCE:      pyutLink.sourceCardinality,
            XmlConstants.ATTR_CARDINALITY_DESTINATION: pyutLink.destinationCardinality,
            XmlConstants.ATTR_BIDIRECTIONAL:           str(pyutLink.getBidir()),
            XmlConstants.ATTR_SOURCE_ID:               str(srcLinkId),
            XmlConstants.ATTR_DESTINATION_ID:          str(destLinkId),
        })
        pyutLinkElement: Element = SubElement(oglLinkElement, XmlConstants.ELEMENT_PYUT_LINK, attrib=attributes)

        return pyutLinkElement

    def pyutInterfaceToXml(self, pyutInterface: PyutInterface, interface2Element: Element) -> Element:

        classId: int = self._idFactory.getID(pyutInterface)
        attributes: ElementAttributes = ElementAttributes({
            XmlConstants.ATTR_ID:          str(classId),
            XmlConstants.ATTR_NAME:        pyutInterface.name,
            XmlConstants.ATTR_DESCRIPTION: pyutInterface.description
        })
        pyutInterfaceElement: Element = SubElement(interface2Element, XmlConstants.ELEMENT_MODEL_INTERFACE, attrib=attributes)

        for method in pyutInterface.methods:
            self._pyutMethodToXml(pyutMethod=method, pyutClassElement=pyutInterfaceElement)

        for className in pyutInterface.implementors:
            self.logger.debug(f'implementing className: {className}')
            self._pyutImplementorToXml(className, pyutInterfaceElement)

        return pyutInterfaceElement

    def pyutNoteToXml(self, pyutNote: PyutNote, oglNoteElement: Element) -> Element:

        noteId:       int = self._idFactory.getID(pyutNote)
        content:      str = pyutNote.content
        fixedContent: str  = content.replace(osLineSep, PyutToXml.END_OF_LINE_MARKER)
        if pyutNote.fileName is None:
            pyutNote.fileName = ''

        attributes: ElementAttributes = ElementAttributes({
            XmlConstants.ATTR_ID:       str(noteId),
            XmlConstants.ATTR_CONTENT:  fixedContent,
            XmlConstants.ATTR_FILENAME: pyutNote.fileName,
        })
        pyutNoteElement: Element = SubElement(oglNoteElement, XmlConstants.ELEMENT_PYUT_NOTE, attrib=attributes)

        return pyutNoteElement

    def pyutTextToXml(self, pyutText: PyutText, oglTextElement: Element) -> Element:

        textId:       int = self._idFactory.getID(pyutText)
        content:      str = pyutText.content
        fixedContent: str  = content.replace(osLineSep, PyutToXml.END_OF_LINE_MARKER)

        attributes: ElementAttributes = ElementAttributes({
            XmlConstants.ATTR_ID:       str(textId),
            XmlConstants.ATTR_CONTENT:  fixedContent,
        })
        pyutTextElement: Element = SubElement(oglTextElement, XmlConstants.ELEMENT_PYUT_NOTE, attrib=attributes)

        return pyutTextElement

    def pyutActorToXml(self, pyutActor: PyutActor, oglActorElement: Element) -> Element:

        actorId:  int = self._idFactory.getID(pyutActor)
        fileName: str = pyutActor.fileName
        if fileName is None:
            fileName = ''

        attributes: ElementAttributes = ElementAttributes({
            XmlConstants.ATTR_ID:       str(actorId),
            XmlConstants.ATTR_NAME:     pyutActor.name,
            XmlConstants.ATTR_FILENAME: fileName,
        })
        pyutActorElement: Element = SubElement(oglActorElement, XmlConstants.ELEMENT_PYUT_ACTOR, attributes)

        return pyutActorElement

    def pyutUseCaseToXml(self, pyutUseCase: PyutUseCase, oglUseCaseElement: Element) -> Element:

        useCaseId: int = self._idFactory.getID(pyutUseCase)
        fileName:  str = pyutUseCase.fileName
        if fileName is None:
            fileName = ''

        attributes: ElementAttributes = ElementAttributes({
            XmlConstants.ATTR_ID:       str(useCaseId),
            XmlConstants.ATTR_NAME:     pyutUseCase.name,
            XmlConstants.ATTR_FILENAME: fileName
        })
        pyutUseCaseElement: Element = SubElement(oglUseCaseElement, XmlConstants.ELEMENT_PYUT_USE_CASE, attributes)

        return pyutUseCaseElement

    def pyutSDInstanceToXml(self, pyutSDInstance: PyutSDInstance, oglSDInstanceElement: Element) -> Element:

        sdInstanceId: int     = self._idFactory.getID(pyutSDInstance)
        attributes: ElementAttributes = ElementAttributes({
            XmlConstants.ATTR_ID:               str(sdInstanceId),
            XmlConstants.ATTR_INSTANCE_NAME:    pyutSDInstance.instanceName,
            XmlConstants.ATTR_LIFE_LINE_LENGTH: str(pyutSDInstance.instanceLifeLineLength),
        })

        pyutSDInstanceElement: Element = SubElement(oglSDInstanceElement, XmlConstants.ELEMENT_PYUT_SD_INSTANCE, attrib=attributes)

        return pyutSDInstanceElement

    def pyutSDMessageToXml(self, pyutSDMessage: PyutSDMessage, oglSDMessageElement: Element) -> Element:

        sdMessageId: int = self._idFactory.getID(pyutSDMessage)

        srcInstance: PyutSDInstance = pyutSDMessage.getSource()
        dstInstance: PyutSDInstance = pyutSDMessage.getDestination()

        idSrc: int = self._idFactory.getID(srcInstance)
        idDst: int = self._idFactory.getID(dstInstance)

        attributes: ElementAttributes = ElementAttributes({
            XmlConstants.ATTR_ID:                        str(sdMessageId),
            XmlConstants.ATTR_MESSAGE:                   pyutSDMessage.message,
            XmlConstants.ATTR_SOURCE_TIME:               str(pyutSDMessage.sourceY),
            XmlConstants.ATTR_DESTINATION_TIME:          str(pyutSDMessage.destinationY),
            XmlConstants.ATTR_SD_MESSAGE_SOURCE_ID:      str(idSrc),
            XmlConstants.ATTR_SD_MESSAGE_DESTINATION_ID: str(idDst),
        })

        pyutSDMessageElement: Element = SubElement(oglSDMessageElement, XmlConstants.ELEMENT_PYUT_SD_MESSAGE, attrib=attributes)

        return pyutSDMessageElement

    def _pyutMethodToXml(self, pyutMethod: PyutMethod, pyutClassElement: Element) -> Element:
        """
        Exporting a PyutMethod to an Element

        Args:
            pyutMethod:        Method to serialize
            pyutClassElement:  xml document

        Returns:
            The new updated element
        """
        attributes = {
            XmlConstants.ATTR_NAME:               pyutMethod.name,
            XmlConstants.ATTR_VISIBILITY:         pyutMethod.visibility.name,
            XmlConstants.ATTR_METHOD_RETURN_TYPE: pyutMethod.returnType.value,
        }
        pyutMethodElement: Element = SubElement(pyutClassElement, XmlConstants.ELEMENT_PYUT_METHOD, attrib=attributes)
        for modifier in pyutMethod.modifiers:
            attributes = {
                XmlConstants.ATTR_NAME: modifier.name,
            }
            SubElement(pyutMethodElement, XmlConstants.ELEMENT_MODEL_MODIFIER, attrib=attributes)
        self._pyutSourceCodeToXml(pyutMethod.sourceCode, pyutMethodElement)

        for pyutParameter in pyutMethod.parameters:
            self._pyutParameterToXml(pyutParameter, pyutMethodElement)
        # pyutMethodElement: Element = xmlDoc.createElement(XmlConstants.ELEMENT_MODEL_METHOD)
        #
        # pyutMethodElement.setAttribute(XmlConstants.ATTR_NAME, pyutMethod.name)
        #
        # visibility: PyutVisibilityEnum = pyutMethod.getVisibility()
        # visName:    str                = self.__safeVisibilityToName(visibility)
        #
        # if visibility is not None:
        #     pyutMethodElement.setAttribute(XmlConstants.ATTR_VISIBILITY, visName)
        #
        # for modifier in pyutMethod.modifiers:
        #     xmlModifier: Element = xmlDoc.createElement(XmlConstants.ELEMENT_MODEL_MODIFIER)
        #     xmlModifier.setAttribute(XmlConstants.ATTR_NAME, modifier.name)
        #     pyutMethodElement.appendChild(xmlModifier)
        #
        # if pyutMethod.returnType is not None:
        #     xmlReturnType: Element = xmlDoc.createElement(XmlConstants.ELEMENT_MODEL_RETURN)
        #     xmlReturnType.setAttribute(XmlConstants.ATTR_TYPE, str(pyutMethod.returnType))
        #     pyutMethodElement.appendChild(xmlReturnType)
        #
        # for param in pyutMethod.parameters:
        #     pyutMethodElement.appendChild(self._pyutParamToDom(param, xmlDoc))
        #
        # codeRoot: Element = self._pyutSourceCodeToDom(pyutMethod.sourceCode, xmlDoc)
        # pyutMethodElement.appendChild(codeRoot)
        # return pyutMethodElement
        return pyutMethodElement

    def _pyutClassCommonAttributes(self, classCommon: PyutClassCommon):

        attributes = {
            XmlConstants.ATTR_DESCRIPTION: classCommon.description
        }
        return attributes

    def _pyutSourceCodeToXml(self, sourceCode: SourceCode, pyutMethodElement: Element):

        codeRoot: Element = SubElement(pyutMethodElement, XmlConstants.ELEMENT_MODEL_SOURCE_CODE)

        for code in sourceCode:
            codeElement: Element = SubElement(codeRoot, XmlConstants.ELEMENT_MODEL_CODE)
            codeElement.text = code

        return codeRoot

    def _pyutParameterToXml(self, pyutParameter: PyutParameter, pyutMethodElement: Element) -> Element:

        attributes = {
            XmlConstants.ATTR_NAME:          pyutParameter.name,
            XmlConstants.ATTR_TYPE:          pyutParameter.type.value,
            XmlConstants.ATTR_DEFAULT_VALUE: pyutParameter.defaultValue,
        }
        pyutParameterElement: Element = SubElement(pyutMethodElement, XmlConstants.ELEMENT_MODEL_PYUT_PARAMETER, attrib=attributes)

        return pyutParameterElement

    def _pyutFieldToXml(self, pyutField: PyutField, pyutClassElement: Element) -> Element:
        """
        Serialize a PyutField to an Element

        Args:
            pyutField:         The PyutField to serialize
            pyutClassElement: The Pyut Class element to update

        Returns:
            The new updated element
        """
        attributes = {
            XmlConstants.ATTR_NAME:          pyutField.name,
            XmlConstants.ATTR_VISIBILITY:    pyutField.visibility.name,
            XmlConstants.ATTR_TYPE:          pyutField.type.value,
            XmlConstants.ATTR_DEFAULT_VALUE: pyutField.defaultValue,
        }
        pyutFieldElement: Element = SubElement(pyutClassElement, XmlConstants.ELEMENT_MODEL_PYUT_FIELD, attrib=attributes)

        return pyutFieldElement

    def _pyutImplementorToXml(self, className: ClassName, xmlDoc: Element) -> Element:

        # root: Element = xmlDoc.createElement(XmlConstants.ELEMENT_IMPLEMENTOR)
        # root.setAttribute()
        attributes: ElementAttributes = ElementAttributes({
            XmlConstants.ATTR_IMPLEMENTING_CLASS_NAME: className,
        })
        implementorElement: Element = SubElement(xmlDoc, XmlConstants.ELEMENT_IMPLEMENTOR, attrib=attributes)
        return implementorElement
